import json


class OriginalAPIException(Exception):
    def __init__(self, text: str, status_code: int) -> None:
        self.response_text = text
        self.status_code = status_code
        self.json_response = False

        try:
            full_error = json.loads(text)
            self.error_type = full_error["error"]["type"]
            self.error_detail = full_error["error"]["detail"]
            self.json_response = True
        except ValueError:
            pass

    def __str__(self) -> str:
        if self.json_response:
            return f"Original error code {self.status_code}: type: {self.error_type}: {self.error_detail}"
        else:
            return f"Original error HTTP code: {self.status_code}"



import abc
import os
from typing import Any, Awaitable, Dict, Union

import jwt

from original_sdk.types.original_response import OriginalResponse

DEFAULT_BASE_URL = "https://api.getoriginal.com"
DEFAULT_API_VERSION = "v1"


class BaseOriginalClient(abc.ABC):
    def __init__(
        self, api_key: str, api_secret: str, timeout: float = 6.0, **options: Any
    ):
        self.api_key = api_key
        self.api_secret = api_secret
        self.timeout = timeout

        if os.getenv("ORIGINAL_TIMEOUT"):
            self.timeout = float(os.environ["ORIGINAL_TIMEOUT"])

        self.options = options
        self.base_url = DEFAULT_BASE_URL
        self.api_version = DEFAULT_API_VERSION

        if options.get("base_url"):
            self.base_url = options["base_url"]
        elif os.getenv("ORIGINAL_URL"):
            self.base_url = os.environ["ORIGINAL_URL"]

        if options.get("api_version"):
            self.api_version = options["api_version"]
        elif os.getenv("API_VERSION"):
            self.api_version = os.environ["API_VERSION"]

        self.base_url = self.base_url.rstrip("/")

        self.token = jwt.encode(
            {"resource": "*", "action": "*", "user_id": "*", "api_key": api_key},
            api_secret,
        )

    def get_default_params(self) -> Dict[str, str]:
        return {"api_key": self.api_key, "api_secret": self.api_secret}

    def create_token(self) -> str:
        return jwt.encode(
            {"resource": "*", "action": "*", "user_id": "*", "api_key": self.api_key},
            self.api_secret,
        )

    def create_search_params(
        self,
        filter_conditions: Dict,
        query: Union[str, Dict],
        **options: Any,
    ) -> Dict[str, Any]:
        params = options.copy()
        if isinstance(query, str):
            params.update({"query": query})
        else:
            params.update({"message_filter_conditions": query})

        params.update({"filter_conditions": filter_conditions})

        return params

    @abc.abstractmethod
    def create_user(
        self, email: str, client_id: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Create an Original user.
        """
        pass

    @abc.abstractmethod
    def get_user(
        self, uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original user.

        :param uid: the user uid
        :return:
        """
        pass

    @abc.abstractmethod
    def get_user_by_email(
        self, email: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Gets a user by email

        :param email: the users email
        :return:
        """
        pass

    @abc.abstractmethod
    def get_user_by_client_id(
        self, client_id: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Gets a user by client_id

        :param client_id: the users email
        :return:
        """
        pass

    @abc.abstractmethod
    def get_collection(
        self, uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original collection.

        :param uid: the collection uid
        :return:
        """
        pass

    @abc.abstractmethod
    def create_asset(
        self, **asset_data: Any
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Create an Original asset.

        :param asset_data: the asset data
        :return:
        """
        pass

    @abc.abstractmethod
    def edit_asset(
        self, asset_uid: str, **asset_data: Any
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Update an Original asset.

        :param asset_uid: the asset uid
        :param asset_data: the asset data
        :return:
        """
        pass

    @abc.abstractmethod
    def get_asset(
        self, asset_uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original asset.

        :param asset_uid: the asset uid
        :return:
        """
        pass

    @abc.abstractmethod
    def get_assets_by_user_uid(
        self, app_user_uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original asset by owner uid.

        :param app_user_uid: the app user uid
        :return:
        """
        pass

    @abc.abstractmethod
    def create_transfer(
        self, **transfer_data: Any
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Create an Original transfer.

        :param transfer_data: the transfer data
        :return:
        """
        pass

    @abc.abstractmethod
    def get_transfer(
        self, transfer_uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original transfer.

        :param transfer_uid: the transfer uid
        :return:
        """
        pass

    @abc.abstractmethod
    def get_transfers_by_user_uid(
        self, app_user_uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original transfer by user uid.

        :param app_user_uid: the app user uid
        :return:
        """
        pass

    @abc.abstractmethod
    def create_burn(
        self, **burn_data: Any
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Create an Original burn.

        :param burn_data: the burn data
        :return:
        """
        pass

    @abc.abstractmethod
    def get_burn(
        self, burn_uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original burn.

        :param burn_uid: the burn uid
        :return:
        """
        pass

    @abc.abstractmethod
    def get_burns_by_user_uid(
        self, app_user_uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original burn by user uid.

        :param app_user_uid: the app user uid
        :return:
        """
        pass

    @abc.abstractmethod
    def get_deposit(
        self, user_uid: str
    ) -> Union[OriginalResponse, Awaitable[OriginalResponse]]:
        """
        Get an Original deposit by user uid.

        :param app_user_uid: the app user uid
        :return:
        """
        pass



import json
from typing import Any, Callable, Dict

import requests

from .__pkg__ import __version__
from .base.client import BaseOriginalClient
from .base.exceptions import OriginalAPIException
from .types.original_response import OriginalResponse


def get_user_agent() -> str:
    return f"original_sdk-python-client-{__version__}"


def get_default_header() -> Dict[str, str]:
    base_headers = {
        "Content-type": "application/json",
        "X-ORIGINAL-Client": get_user_agent(),
    }
    return base_headers


class OriginalClient(BaseOriginalClient):
    def __init__(
        self, api_key: str, api_secret: str, timeout: float = 6.0, **options: Any
    ):
        super().__init__(
            api_key=api_key, api_secret=api_secret, timeout=timeout, **options
        )
        self.session = requests.Session()
        self.session.mount("http://", requests.adapters.HTTPAdapter(max_retries=1))
        self.session.mount("https://", requests.adapters.HTTPAdapter(max_retries=1))

    def set_http_session(self, session: requests.Session) -> None:
        """
        You can use your own `requests.Session` instance. This instance
        will be used for underlying HTTP requests.
        """
        self.session = session

    def _parse_response(self, response: requests.Response) -> OriginalResponse:
        try:
            parsed_result = json.loads(response.text) if response.text else {}
        except ValueError:
            raise OriginalAPIException(response.text, response.status_code)
        if response.status_code >= 399:
            raise OriginalAPIException(response.text, response.status_code)

        return OriginalResponse(
            parsed_result, dict(response.headers), response.status_code
        )

    def _make_request(
        self,
        method: Callable[..., requests.Response],
        relative_url: str,
        params: Dict = None,
        data: Any = None,
    ) -> OriginalResponse:
        params = params or {}
        data = data or {}
        serialized = None
        default_params = self.get_default_params()
        default_params.update(params)
        headers = get_default_header()
        headers["Authorization"] = f"Bearer {self.token}"
        headers["X-API-KEY"] = self.api_key

        url = f"{self.base_url}/{self.api_version}/{relative_url}"
        if method.__name__ in ["post", "put", "patch"]:
            serialized = json.dumps(data)

        response = method(
            url,
            data=serialized,
            headers=headers,
            params=default_params,
            timeout=self.timeout,
        )
        return self._parse_response(response)

    def put(
        self, relative_url: str, params: Dict = None, data: Any = None
    ) -> OriginalResponse:
        return self._make_request(self.session.put, relative_url, params, data)

    def post(
        self, relative_url: str, params: Dict = None, data: Any = None
    ) -> OriginalResponse:
        return self._make_request(self.session.post, relative_url, params, data)

    def get(self, relative_url: str, params: Dict = None) -> OriginalResponse:
        return self._make_request(self.session.get, relative_url, params, None)

    def delete(self, relative_url: str, params: Dict = None) -> OriginalResponse:
        return self._make_request(self.session.delete, relative_url, params, None)

    def patch(
        self, relative_url: str, params: Dict = None, data: Any = None
    ) -> OriginalResponse:
        return self._make_request(self.session.patch, relative_url, params, data)

    def create_user(self, email: str, client_id: str) -> OriginalResponse:
        return self.post("user", data={"email": email, "client_id": client_id})

    def get_user(self, user_id: str) -> OriginalResponse:
        return self.get(f"user/{user_id}")

    def get_user_by_email(self, email: str) -> OriginalResponse:
        return self.get("user", params={"email": email})

    def get_user_by_client_id(self, client_id: str) -> OriginalResponse:
        return self.get("user", params={"client_id": client_id})

    def get_collection(self, uid: str) -> OriginalResponse:
        return self.get(f"collection/{uid}")

    def create_asset(self, **asset_data: Any) -> OriginalResponse:
        return self.post("asset", data=asset_data)

    def edit_asset(self, asset_uid: str, **asset_data: Any) -> OriginalResponse:
        return self.put(f"asset/{asset_uid}", data=asset_data)

    def get_asset(self, uid: str) -> OriginalResponse:
        return self.get(f"asset/{uid}")

    def get_assets_by_user_uid(self, user_uid: str) -> OriginalResponse:
        return self.get("asset", params={"user_uid": user_uid})

    def create_transfer(self, **transfer_data: Any) -> OriginalResponse:
        return self.post("transfer", data=transfer_data)

    def get_transfer(self, uid: str) -> OriginalResponse:
        return self.get(f"transfer/{uid}")

    def get_transfers_by_user_uid(self, user_uid: str) -> OriginalResponse:
        return self.get("transfer", params={"user_uid": user_uid})

    def create_burn(self, **burn_data: Any) -> OriginalResponse:
        return self.post("burn", data=burn_data)

    def get_burn(self, uid: str) -> OriginalResponse:
        return self.get(f"burn/{uid}")

    def get_burns_by_user_uid(self, user_uid: str) -> OriginalResponse:
        return self.get("burn", params={"user_uid": user_uid})

    def get_deposit(self, user_uid: str) -> OriginalResponse:
        return self.get("deposit", params={"user_uid": user_uid})



from datetime import datetime, timezone
from typing import Any, Dict, Optional

from original_sdk.types.rate_limit import RateLimitInfo


class OriginalResponse(dict):
    """
    A custom dictionary where you can access the response data the same way
    as a normal dictionary. Additionally, we expose some methods to access other metadata.

    ::

        resp = client.get_user()
        print(resp["data"]["uid"])
        # "xxxxxxxx..."
        rate_limit = resp.rate_limit()
        print(rate_limit.remaining)
        # 99
        headers = resp.headers()
        print(headers["content-type"])
        # "application/json;charset=utf-8"
        status_code = resp.status_code()
        print(status_code)
        # 200
    """

    def __init__(
        self, response_dict: Dict[str, Any], headers: Dict[str, Any], status_code: int
    ) -> None:
        self.__headers = headers
        self.__status_code = status_code
        self.__rate_limit: Optional[RateLimitInfo] = None
        limit, remaining, reset = (
            headers.get("x-ratelimit-limit"),
            headers.get("x-ratelimit-remaining"),
            headers.get("x-ratelimit-reset"),
        )
        if limit and remaining and reset:
            self.__rate_limit = RateLimitInfo(
                limit=int(self._clean_header(limit)),
                remaining=int(self._clean_header(remaining)),
                reset=datetime.fromtimestamp(
                    float(self._clean_header(reset)), timezone.utc
                ),
            )

        super(OriginalResponse, self).__init__(response_dict)

    def _clean_header(self, header: str) -> int:
        try:
            values = (v.strip() for v in header.split(","))
            return int(next(v for v in values if v))
        except ValueError:
            return 0

    def rate_limit(self) -> Optional[RateLimitInfo]:
        """Returns the ratelimit info of your API operation."""
        return self.__rate_limit

    def headers(self) -> Dict[str, Any]:
        """Returns the headers of the response."""
        return self.__headers

    def status_code(self) -> int:
        """Returns the HTTP status code of the response."""
        return self.__status_code
